# -*- coding: utf-8 -*-
"""
    simplewiki
    ~~~~~~~~~~

    Very simple wiki application based on Genshi, Werkzeug and SQLAlchemy.
    Additionally the creoleparser is used for the wiki markup.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
from .application import SimpleWiki
