<?python hide_navigation = True ?>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xi="http://www.w3.org/2001/XInclude"
      xmlns:py="http://genshi.edgewall.org/"><xi:include href="layout.html" />
  <head>
    <title>Action Not Found</title>
  </head>
  <body>
    <h1>Action “${action}” Not Found</h1>
    <p>The requested action does not exist.</p>
    <p>Try to <a href="?">access the same URL</a> without parameters.</p>
  </body>
</html>
