<?python
  hide_navigation = True
?>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xi="http://www.w3.org/2001/XInclude"
      xmlns:py="http://genshi.edgewall.org/"><xi:include href="layout.html" />
  <head>
    <title>Index</title>
  </head>
  <body>
    <h1>Index</h1>
    <py:for each="letter, pages in letters">
      <h2 id="${letter}"><a href="#${letter}">${letter}</a></h2>
      <ul>
        <li py:for="page in pages"><a href="${href(page.name)}">${page.title}</a></li>
      </ul>
    </py:for>
  </body>
</html>
